#ifndef _VOLUMEFILE_H_
#define _VOLUMEFILE_H_

#include "global_define.h"
#include "define.h"
#include "queue.h"

#include <qstring.h>
#include <qcolor.h>
#include <qtextstream.h>


#define ISVOLUME	0x4D56		// volume signature is 0x4D56 : 'VM' => 4D:M 56:V

#define SURFACE1	0x01
#define SURFACE2	0x02
#define SURFACE3	0x04
#define SURFACE4	0x08
#define SURFACE5	0x10
#define SURFACE6	0x20

typedef struct{
	WORD   vfVersion;		// file version
	WORD   vfType;			// file type "VX"
	DWORD  vfSize;			// file size
} VOLUMEFILEHEADER;

typedef struct{
	WORD   viSize;			// structure size

	WORD   viWidth;			// volume width   : x-direction
	WORD   viHeight;		// volume height  : y-direction
	WORD   viDepth;			// volume depth   : z-direction

	float  fDx;				// voxel size in width-direction  (= element width)
	float  fDy;				// voxel size in height-direction (= element height)
	float  fDz;				// voxel size in depth-direction  (= element depth)

	float  fOriginDx;		// original voxel size in width-direction
	float  fOriginDy;		// original voxel size in height-direction
	float  fOriginDz;		// original voxel size in depth-direction

//	RGB    rgbThreshold;

//	DWORD  viTotalNumVoxel;	// total number of voxel in volume
} VOLUMEINFOHEADER;

typedef struct{
	WORD   wValue;
	BYTE   byteFlag;
} VOXEL;

typedef struct{
	VOLUMEFILEHEADER VolumeFileHeader;
	VOLUMEINFOHEADER VolumeInfoHeader;

	WORD  *wDensity;
	BYTE  *byteFlag;

	// VOXEL *pVoxel;
} VOLUMESTRUCT;


enum { SRF_STL_ASCII=1, SRF_STL_BINARY, SRF_ACAD_SCRIPT, SRF_ACAD_DXF };


class MDIWindow;


class CVolume
{
	VOLUMESTRUCT m_Volume;

	// number of feasible voxel
	// with respect to current THRESHOLD value
	DWORD m_dwNumCurrentFeaVoxel;
	 
	// number of feasible surface
	// with respect to current THRESHOLD value
	// [0] : 1st & 2nd direction
	// [1] : 3rd & 4th direction
	// [2] : 5th & 6th direction
	DWORD m_dwCurrentFeaSurface[3];

	// number of fragment (bone)
	DWORD m_dwNumCurrentBone;

	// number of filtered fragment (bone)
	DWORD m_dwNumFilteredBone;

	// number of voxel
	// accorading to density level from 0 to 255
	DWORD m_dwNumVoxel[256];

	// number of accumulated feasible voxel
	// accorading to density level from 0 to 255
	DWORD m_dwNumFeaVoxel[256];

	// bone surface area [mm^2]
	// accorading to density level from 0 to 255
	float m_fBoneSurfaceArea[256];

	// bone volume [mm^3]
	// accorading to density level from 0 to 255
	float m_fBoneVolume[256];

	// number of bone per density from 0 to 255;
	// accorading to density level from 0 to 255
	DWORD m_dwNumBone[256];

	// which voxel is bone
	// true : white, false : black
	short m_nWhiteIsBone;
	
	WORD *m_pMinThreshold;
	WORD *m_pMaxThreshold;

	QString m_strFileName;

	short m_nDicomFlag;
	WORD m_wMinImageValue;
	WORD m_wMaxImageValue;

	bool m_bOpenFlag;

	MDIWindow *m_pMDIWindow;

	// number of voxel need to filtering
	DWORD m_nNumFilterVoxel;

	bool m_bMeshFiltered;

	// material type
	bool m_bHomogeneousPossion;	// if true, homogeneous, otherwise heterogeneous
	bool m_bHomogeneousDensity;
	bool m_bHomogeneousYoung;

	// material property lookup table
	float m_fPossionLUT[256];
	float m_fDensityLUT[256];
	float m_fYoungLUT[256];

	bool m_bSurfaceContact;		// if true, use surface contact, otherwise use point contact
	
private:
	bool setVolumeInfoHeader( KEYWORD key );
	bool setVolumeStructFromImage( KEYWORD key );
	bool setVolumeStructFromDicom( KEYWORD key );

	bool evaluateSurrVoxel( DWORD id, CQueue &queue );	// evaluate surrounding VOXEL

	// expport to surface
	bool writeToSTLAsciiFormat( QTextStream &ts, SRF_NODE node[4], int face );
	bool writeToSTLBinaryFormat( QDataStream &ds, SRF_NODE node[4], int face );
	bool writeToAcadScriptFormat( QTextStream &ts, SRF_NODE node[4], WORD value );
	bool writeToAcadDxfFormat( QTextStream &ts, SRF_NODE node[4] );

public:
	/*** constructor & destructor ***/
	CVolume( MDIWindow *mdi );
	~CVolume();

	MDIWindow *mdiWindow();


	/*** filo I/O info ***/
	bool   open( QString fileName );				// open the VOLUME file
	void   close();
	bool   init( KEYWORD key );					// initialize from the KEYWORD file
	bool   save( QString fileName );				// save VOLUME data to disk
	bool   isOpen();


	/*** voxel info ***/
	void   setVoxelWidth( float width );		// set VOXEL width  [x : mm] (= element width)
	void   setVoxelHeight( float height );		// set VOXEL height [y : mm] (= element height)
	void   setVoxelDepth( float depth );		// set VOXEL depth  [z : mm] (= element depth)

	float  voxelWidth();						// get VOXEL width  [x : mm] (= element width)
	float  voxelHeight();						// get VOXEL height [y : mm] (= element height)
	float  voxelDepth();						// get VOXEL depth  [z : mm] (= element depth)

	float  voxelOriginWidth();					// get Original VOXEL width
	float  voxelOriginHeight();					// get Original VOXEL height
	float  voxelOriginDepth();					// get Original VOXEL depth
	
	float  voxelVolume();						// get VOXEL volume [mm^3]

	bool   voxelPosition( DWORD id, POSITION_THREED &posPosi );
												// get position of the voxel

	//VOXEL  voxel( DWORD id );					// get VOXEL

	WORD   voxelValue( DWORD id );				// get VOXEL value
	BYTE   voxelColor( DWORD id );				// get VOXEL color fo display
	BYTE   voxelFlag( DWORD id );				// get VOXEL flag for surface
	void   setMask( DWORD id, bool flag );
	bool   isMaskOn( DWORD id );

//	void   setVoxelFlag( DWORD id, BYTE flag )	// set VOXEL byteFlag


	/*** threshold info ***/
	/*
	void   setThresholdRgb( RGB rgb );			// set ThresholdRgb

	RGB    thresholdRgb();						// get ThresholdRgb
	float  thresholdGrayValue();				// get ThresholdRgb as gray scale
	*/
	WORD   minThreshold( int id );						// get Min Threshold Value
	void   setMinThreshold( int id, WORD minValue );	// set Min Threshold Value
	void   setMinThreshold( int id, RGB rgbMin );

	WORD   maxThreshold( int id );						// get Max Threshold Value
	void   setMaxThreshold( int id, WORD maxValue );	// set Max Threshold Value
	void   setMaxThreshold( int id, RGB rgbMax );
		

	/*** get volume info ***/
	WORD   volumeWidth();						// get width of VOLUME  [x]
	WORD   volumeHeight();						// get height of VOLUME [y]
	WORD   volumeDepth();						// get depth of VOLUME  [z]

	float  volumeVolume();						// get volume of VOLUME [mm^3]
	float  volumeSurfaceArea();					// get surface area of VOLUME [mm^2]
	float  feaVolume();							// get bone volume of VOLUME [mm^3]
												// with respect to current segmentation value
	float  feaSurfaceArea();					// get bone surface area of VOLUME [mm^2]
												// with respect to current THRESHOLD value
	DWORD  totalNumVoxel();						// get total number of VOXEL in VOLUME


	/*** etc info ***/
	DWORD  numCurrentFeaVoxel();				// get number of feasible voxel
												// with respect to current segmentation value
	DWORD  numCurrentBone();					// get number of current fragment (bone)
	DWORD  numFilteredBone();					// get number of filtered fragment (bone)

	DWORD  numVoxel( int id );					// get number of voxel
												// according to density level from 0 to 255
	DWORD  numFeaVoxel( int id );				// get number of accumulated feasible voxel
												// according to density level from 0 to 255
	float  boneSurfaceArea( int id );			// get bone surface area [mm^2]
												// according to density level from 0 to 255
	float  boneVolume( int id );				// get bone volume [mm^3]
												// according to density level from 0 to 255
	DWORD  numBone( int id );					// get number of bone
												// according to density level from 0 to 255

	short  isWhiteIsBone() { return m_nWhiteIsBone; }
	bool   isDicom();

	WORD   minImageValue() { return m_wMinImageValue; }
	WORD   maxImageValue() { return m_wMaxImageValue; }

	WORD  *density();

	
	/*** calculation info ***/
	bool   analysisVolume( int start=0, int end=255 );
												// analysis VOLUME from start to end

	void   calculateNumVoxel();					// calculate number of voxel
												// according to density level from 0 to 255

	bool   calculateNumCurrentBone( bool bfilter=false );
												// calculate number of bone in VOLUME
												// with respect to current segmentation value

	bool   isMeshFiltered();
	void   setMeshFilter( bool filter );
	
	bool   findCurrentVisibleSurface( bool reset=false );
												// find visible surface
												// with respect to current segmentation value


	/*** filter info ***/
	void   setNumFilterVoxel( DWORD voxel );
	DWORD  numFilterVoxel();


	/*** volume file name info ***/
	QString volumeFileName();					// get volume file name

	
	/*** export finite element ***/
	bool   exportToAnsys( QString fileName );

	void   setHomogeneousPossion( bool homo );
	void   setHomogeneousDensity( bool homo );
	void   setHomogeneousYoung( bool homo );

	void   setPossionLUT( int idx, float value );
	void   setDensityLUT( int idx, float value );
	void   setYoungLUT( int idx, float value );

	bool   isSurfaceContact();
	void   setSurfaceContact( bool flag );


	/*** export surface ***/
	bool   exportToSurface( QString fileName, int type );


public:
	// static member function
	static DWORD  floatToDWORD( float value_in );
	static float  rgbToGray( QRgb rgb );
//	static int    checkFileExistence( char string[256] );
//	static bool   storeToBMP( unsigned char *image, int width, int height, char *filename );
};

#endif // _VOLUMEFILE_H_
