/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "define.h"
#include "ImportStackWizard.h"
#include "MainWindow.h"
#include "DicomFile.h"

#include <qimage.h>
#include <qpoint.h>
#include <qsize.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qfiledialog.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qscrollview.h>
#include <qstringlist.h>
#include <qvalidator.h>
#include <qlabel.h>
#include <qpen.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qmessagebox.h>
#include <fstream.h>		// c++



////////////////////////////////////////////////////////////////////////////
//
// CImportStackWizard
//

CImportStackWizard::CImportStackWizard( QWidget *parent, const char *name, bool modal, WFlags fl )
	: QWizard( parent, name, modal, fl )
{
//	m_pImage = NULL;
	m_pImage = new QImage;

	setupSelectionPage();
	setupImageSizePage();
	setupVoxelInfoPage();
	setupROIPage();
	setupBoneColorPage();

	connect( cancelButton(), SIGNAL( clicked() ),
		this, SLOT( reject() ) );
	connect( finishButton(), SIGNAL( clicked() ),
		this, SLOT( slotFinishWizard() ) );
} /* CImportStackWizard */

CImportStackWizard::~CImportStackWizard()
{
	if ( m_pImage )
	{
		delete m_pImage;
		m_pImage = NULL;
	}
	
	if ( m_pFileSelectionDlg )
	{
		delete m_pFileSelectionDlg;
		m_pFileSelectionDlg = NULL;
	}
	
	if ( m_pImageSizeDlg )
	{
		delete m_pImageSizeDlg;
		m_pImageSizeDlg = NULL;
	}

	if ( m_pVoxelInfoDlg )
	{
		delete m_pVoxelInfoDlg;
		m_pVoxelInfoDlg = NULL;
	}

	if ( m_pROIDlg )
	{
		delete m_pROIDlg;
		m_pROIDlg = NULL;
	}

	if ( m_pBoneColorDlg )
	{
		delete m_pBoneColorDlg;
		m_pBoneColorDlg = NULL;
	}
} /* ~CImportStackWizard */

void CImportStackWizard::showPage( QWidget *page )
{
    QWizard::showPage( page );

	if ( page == (QWidget *) m_pFileSelectionDlg )
	{
		m_pFileSelectionDlg->init();
	}
	else if ( page == (QWidget *) m_pImageSizeDlg )
	{
		m_pImageSizeDlg->init();
	}
	else if ( page == (QWidget *) m_pVoxelInfoDlg )
	{
		m_pVoxelInfoDlg->init();
	}
	else if ( page == (QWidget *) m_pROIDlg )
	{
		m_pROIDlg->init();
	}
	else if ( page == (QWidget *) m_pBoneColorDlg )
	{
		m_pBoneColorDlg->init();
		finishButton()->setEnabled( true );
	}
} /* showPage */

void CImportStackWizard::setupSelectionPage()
{
	m_pFileSelectionDlg = new CImportFileSelectionDlg( this );

    addPage( m_pFileSelectionDlg, "File Selection" );

    setNextEnabled( m_pFileSelectionDlg, false );
    setHelpEnabled( m_pFileSelectionDlg, false );
} /* setupSelectionPage */

void CImportStackWizard::setupImageSizePage()
{
	m_pImageSizeDlg = new CImportImageSizeDlg( this );

	addPage( m_pImageSizeDlg, "Image Size" );

	setNextEnabled( m_pImageSizeDlg, true );
	setHelpEnabled( m_pImageSizeDlg, false );
} /* setupImageSizePage */

void CImportStackWizard::setupVoxelInfoPage()
{
	m_pVoxelInfoDlg = new CImportVoxelInfoDlg( this );

	addPage( m_pVoxelInfoDlg, "Voxel Information" );

	setNextEnabled( m_pVoxelInfoDlg, false );
	setHelpEnabled( m_pVoxelInfoDlg, false );
} /* setupVoxelInfoPage */

void CImportStackWizard::setupROIPage()
{
	m_pROIDlg = new CImportROIDlg( this );

	addPage( m_pROIDlg, "Region Of Interest" );

	setNextEnabled( m_pROIDlg, true );
	setHelpEnabled( m_pROIDlg, false );
} /* setupROIPage */

void CImportStackWizard::setupBoneColorPage()
{
	m_pBoneColorDlg = new CImportBoneColorDlg( this );

	addPage( m_pBoneColorDlg, "Bone Voxel Color" );

	setNextEnabled( m_pBoneColorDlg, true );
	setHelpEnabled( m_pBoneColorDlg, false );
} /* setupBoneColorPage */

QStringList CImportStackWizard::fileList()
{
	return m_pFileSelectionDlg->fileList();
} /* fileListBox */

QString CImportStackWizard::imageFileName()
{
	return m_strImageFileName;
} /* imageFileName */

void CImportStackWizard::setImageFileName( QString str )
{
	m_strImageFileName = str;
} /* setImageFileName */

QString CImportStackWizard::imageFormat()
{
	return m_strImageFormat;
} /* imageFileFormat */

void CImportStackWizard::setImageFormat( QString str )
{
	m_strImageFormat = str;
} /* setImageFileFormat */

QImage *CImportStackWizard::image()
{
	if ( m_pImage )
		return m_pImage;
	else 
		return NULL;
} /* image */

void CImportStackWizard::slotFinishWizard()
{
	int w1 = m_pROIDlg->widthDown();
	int w2 = m_pROIDlg->widthUp();
	int h1 = m_pROIDlg->heightDown();
	int h2 = m_pROIDlg->heightUp();
	int d1 = m_pROIDlg->depthDown();
	int d2 = m_pROIDlg->depthUp();


	m_Keyword.strList = m_pFileSelectionDlg->fileList();
	m_Keyword.strImageFormat = m_strImageFormat;

	m_Keyword.regRegion.start_ele.x = w1;
	m_Keyword.regRegion.start_ele.y = h1;
	m_Keyword.regRegion.width = w2 - w1 + 1;
	m_Keyword.regRegion.height = h2 - h1 + 1;
	m_Keyword.regRegion.depth_down = d1;
	m_Keyword.regRegion.depth_up = d2;

	m_Keyword.fVoxelWidth = m_pVoxelInfoDlg->voxelWidth();
	m_Keyword.fVoxelHeight = m_pVoxelInfoDlg->voxelHeight();
	m_Keyword.fVoxelDepth = m_pVoxelInfoDlg->voxelDepth();

	m_Keyword.bWhiteIsBone = m_pBoneColorDlg->whiteIsBone();
} /* slotFinishWizard */

void CImportStackWizard::reject()
{
	QDialog::reject();
} /* reject */


////////////////////////////////////////////////////////////////////////////
//
// CImportFileSelectionDlg
//

CImportFileSelectionDlg::CImportFileSelectionDlg( QWidget *parent, const char *name, bool modal, WFlags fl )
	: CImportFileSelectionDlg_base( parent, name, modal, fl )
{
	m_pParent = (CImportStackWizard *) parent;
	ASSERT( m_pParent );

	m_nOldSortIndex = 0;	// asscending

	// for ListBox
	connect( IDC_LIST_FILE, SIGNAL( currentChanged( QListBoxItem * ) ),
		this, SLOT( slotFileList() ) );

	// for File Type ComboBox
	IDC_COMBO_FILEFORMAT->insertItem( "DICOM" );	// dicom
	for ( unsigned int i = 0; i < QImageIO::outputFormats().count(); i++ )
	{
		QString str = QString( QImageIO::outputFormats().at( i ) );
		IDC_COMBO_FILEFORMAT->insertItem( str ); 
	}
	m_pParent->setImageFormat( "DICOM" );
//	m_pParent->setImageFormat( QImageIO::outputFormats().at( 0 ) );
	connect( IDC_COMBO_FILEFORMAT, SIGNAL( activated( const QString & ) ),
		this, SLOT( slotFileFormat( const QString & ) ) );

	// for Sort ComboBox
	IDC_COMBO_SORT->insertItem( tr( "Ascending" ) );
	IDC_COMBO_SORT->insertItem( tr( "Descending" ) );
	connect( IDC_COMBO_SORT, SIGNAL( activated( int ) ),
		this, SLOT( slotSortList( int ) ) );

	// for File Add, Remove & ClearAll
	connect( IDC_BTN_ADD, SIGNAL( clicked() ), this, SLOT( slotAddFiles() ) );
	connect( IDC_BTN_REMOVE, SIGNAL( clicked() ), this, SLOT( slotRemoveFiles() ) );
	connect( IDC_BTN_CLEARALL, SIGNAL( clicked() ), this, SLOT( slotClearAll() ) );
} /* CImportFileSelectionDlg */

CImportFileSelectionDlg::~CImportFileSelectionDlg()
{
} /* ~CImportFileSelectionDlg */

void CImportFileSelectionDlg::init()
{
	if ( IDC_LIST_FILE->count() != 0 )
		m_pParent->nextButton()->setEnabled( true );
	else
	{
		m_pParent->nextButton()->setEnabled( false );
		IDC_BTN_REMOVE->setEnabled( false );
		IDC_BTN_CLEARALL->setEnabled( false );
	}

	IDC_LBL_DESCRIPTION->setText(
		tr(
"Please select the files containing the slices and specify the proper order."
		) );

} /* init */

QStringList CImportFileSelectionDlg::fileList()
{
	QStringList strList;

	int count = IDC_LIST_FILE->count();

	for ( int i=0 ; i<count ; i++ )
	{
		strList.append( IDC_LIST_FILE->text( i ) );
	}

	return strList;
} /* fileList */

void CImportFileSelectionDlg::slotFileList()
{
	if ( IDC_LIST_FILE->count() != 0 )
	{
		m_pParent->nextButton()->setEnabled( true );
		IDC_BTN_REMOVE->setEnabled( true );
	}
	else
	{
		m_pParent->nextButton()->setEnabled( false );
		IDC_BTN_REMOVE->setEnabled( false );
	}
} /* slotListBox */

void CImportFileSelectionDlg::slotFileFormat( const QString &str )
{
	m_pParent->setImageFormat( str );
} /* slotFileFormat */

void CImportFileSelectionDlg::slotAddFiles()
{
	// file extension
	QString strType, strImage;

	QString tmpStr = IDC_COMBO_FILEFORMAT->currentText();
	if ( tmpStr == "DICOM" )
	{
		strImage = "DICOM";
		strType = "dcm";
	}
	else
	{
		strType = tmpStr.lower();
		strImage = strType.upper();
	}
	QString str;
	if ( strImage == "JPEG" )
		str = strImage + " Image( *.jpeg;*.jpg;*.JPEG;*.JPG )";
	else
		str = strImage + " Image (*." + strType + ";*." + strType.upper() + ")";

	// select file(s)
	QStringList strList;
	strList = QFileDialog::getOpenFileNames( str );

	if ( strList.count() == 0 )
		return;

	// insert selected item to ListBox
	for ( QStringList::Iterator it = strList.begin(); it != strList.end(); ++it )
		IDC_LIST_FILE->insertItem( (*it) );

	bool bSort = true;							// ascending
	if ( IDC_COMBO_SORT->currentItem() == 1 )	// descending
		bSort = false;

	IDC_LIST_FILE->sort( bSort );

	if ( bSort == true )
		IDC_LIST_FILE->setCurrentItem( 0 );
	else
		IDC_LIST_FILE->setCurrentItem( IDC_LIST_FILE->count()-1 );

	m_pParent->setImageFileName( IDC_LIST_FILE->text( 0 ) );
	int nCount = IDC_LIST_FILE->count();
	m_pParent->setNumSlice( nCount );

	if ( nCount != 0 )
	{
		IDC_BTN_REMOVE->setEnabled( true );
		IDC_BTN_CLEARALL->setEnabled( true );
	}
} /* slotAddFiles */

void CImportFileSelectionDlg::slotRemoveFiles()
{
	int nCount = IDC_LIST_FILE->count();
	if ( nCount == 0 ) return;

	for ( int i=nCount-1 ; i>=0 ; i-- )
	{
		if ( IDC_LIST_FILE->isSelected( i ) )
			IDC_LIST_FILE->removeItem( i );
	}

	nCount = IDC_LIST_FILE->count();
	m_pParent->setNumSlice( nCount );

	if ( nCount == 0 )
	{
		IDC_BTN_REMOVE->setEnabled( false );
		IDC_BTN_CLEARALL->setEnabled( false );
	}
} /* slotRemoveFiles */

void CImportFileSelectionDlg::slotClearAll()
{
	IDC_LIST_FILE->clear();
	m_pParent->setNumSlice( 0 );

	m_pParent->nextButton()->setEnabled( false );
	IDC_BTN_REMOVE->setEnabled( false );
	IDC_BTN_CLEARALL->setEnabled( false );
} /* slotClearAll */

void CImportFileSelectionDlg::slotSortList( int index )
{
	if ( m_nOldSortIndex == index ) return;

	bool bSort = true;		// ascending
	if ( index == 1 )		// descending
		bSort = false;

	int currentIndex = IDC_LIST_FILE->currentItem();

	IDC_LIST_FILE->sort( bSort );
	IDC_LIST_FILE->setCurrentItem( currentIndex );

	m_nOldSortIndex = index;
} /* slotSort */


////////////////////////////////////////////////////////////////////////////
//
// CImportImageSizeDlg
//

CImportImageSizeDlg::CImportImageSizeDlg( QWidget *parent, const char *name, bool modal, WFlags fl )
	: CImportImageSizeDlg_base( parent, name, modal, fl )
{
	m_pParent = (CImportStackWizard *) parent;
	ASSERT( m_pParent );

	m_nWidth = -1;
	m_nHeight = -1;

	connect( IDC_BTN_DEFAULT, SIGNAL( clicked() ), this, SLOT( slotSetDefault() ) );

	connect( IDC_SPIN_WIDTH, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotWidthChanged( int ) ) );
	connect( IDC_SPIN_HEIGHT, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotHeightChanged( int ) ) );

	connect( IDC_SLIDER_WIDTH, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotWidthChanged( int ) ) );
	connect( IDC_SLIDER_HEIGHT, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotHeightChanged( int ) ) );
} /* CImportImageSizeDlg */

CImportImageSizeDlg::~CImportImageSizeDlg()
{
} /* ~CImportImageSizeDlg */

void CImportImageSizeDlg::init()
{
	if ( m_nWidth == -1 && m_nHeight == -1 )
		slotSetDefault();
	else
	{
		IDC_SPIN_WIDTH->setValue( m_nWidth );
		IDC_SPIN_HEIGHT->setValue( m_nHeight );

		IDC_SLIDER_WIDTH->setValue( m_nWidth );
		IDC_SLIDER_HEIGHT->setValue( m_nHeight );
	}

	IDC_LBL_DESCRIPTION->setText(
		tr(
"Please specify the size of your data set."
		) );
} /* init */

void CImportImageSizeDlg::slotSetDefault()
{
	QString strFileName = m_pParent->imageFileName();
	QString strFileFormat = m_pParent->imageFormat();

	if ( strFileName == NULL || strFileFormat == NULL )
		return;

	int w, h;

	QImage *pImage = m_pParent->image();
	if ( strFileFormat == "DICOM" )
	{
		CDicomFile df;
		if ( !df.init( strFileName ) )
		{
			pImage = NULL;

			QString warn;
			warn.sprintf( "Can't load %s", strFileName.latin1() );
			QMessageBox::critical( this, "File Load Error", warn );

			return;
		}

		w = df.width();
		h = df.height();

		*pImage = df.image().copy( 0, 0, w, h );	// deep copy

		df.close();
	}
	else
	{
		if ( !pImage->load( strFileName, strFileFormat ) )
		{
			pImage = NULL;

			QString warn;
			warn.sprintf( "Can't load %s", strFileName.latin1() );
			QMessageBox::critical( this, "File Load Error", warn );

			w = h = 0;

			m_pParent->nextButton()->setEnabled( false );
		}
		else
		{
			w = pImage->width();
			h = pImage->height();
		}
	}

	IDC_SPIN_WIDTH->setValue( w );
	IDC_SPIN_HEIGHT->setValue( h );

	IDC_SLIDER_WIDTH->setValue( w );
	IDC_SLIDER_HEIGHT->setValue( h );

	m_nWidth = w;
	m_nHeight = h;
} /* slotSetDEfault */

void CImportImageSizeDlg::slotWidthChanged( int value )
{
	if ( IDC_SLIDER_WIDTH->value() != value )
		IDC_SLIDER_WIDTH->setValue( value );

	if ( IDC_SPIN_WIDTH->value() != value )
		IDC_SPIN_WIDTH->setValue( value );

	m_nWidth = value;
} /* slotWidthChanged */

void CImportImageSizeDlg::slotHeightChanged( int value )
{
	if ( IDC_SLIDER_HEIGHT->value() != value )
		IDC_SLIDER_HEIGHT->setValue( value );

	if ( IDC_SPIN_HEIGHT->value() != value )
		IDC_SPIN_HEIGHT->setValue( value );

	m_nHeight = value;
} /* slotHeightChanged */


////////////////////////////////////////////////////////////////////////////
//
// CImportVoxelInfoDlg
//

CImportVoxelInfoDlg::CImportVoxelInfoDlg( QWidget *parent, const char *name, bool modal, WFlags fl )
	: CImportVoxelInfoDlg_base( parent, name, modal, fl )
{
	m_pParent = (CImportStackWizard *) parent;
	ASSERT( m_pParent );

	m_fWidth = -1.0;
	m_fHeight = -1.0;
	m_fDepth = 0.0;

	QDoubleValidator *validator = new QDoubleValidator( 0.0, 4096.0, 7, this );
	IDC_EDIT_WIDTH->setValidator( validator );
	IDC_EDIT_HEIGHT->setValidator( validator );
	IDC_EDIT_DEPTH->setValidator( validator );

	connect( IDC_EDIT_WIDTH, SIGNAL( textChanged( const QString & ) ),
		this, SLOT( slotValueChanged() ) );
	connect( IDC_EDIT_HEIGHT, SIGNAL( textChanged( const QString & ) ),
		this, SLOT( slotValueChanged() ) );
	connect( IDC_EDIT_DEPTH, SIGNAL( textChanged( const QString & ) ),
		this, SLOT( slotValueChanged() ) );
} /* CImportVoxelInfoDlg */

CImportVoxelInfoDlg::~CImportVoxelInfoDlg()
{
} /* ~CImportVoxelInfoDlg */

void CImportVoxelInfoDlg::init()
{
	if ( m_fWidth == -1.0 && m_fHeight == -1.0 )
	{
		double w = 0, h = 0, d = 0;
		
		QImage *pImage = m_pParent->image();
		if ( pImage == NULL )
		{
			/*
			QMessageBox::information( this, "File Error",
				"Error of Image File" );
			*/

			m_pParent->nextButton()->setEnabled( false );
		}
		else
		{
			int dotX = pImage->dotsPerMeterX();
			int dotY = pImage->dotsPerMeterY();

			if ( dotX != 0 )
				w = (float) ( pImage->width() / dotX );
			if ( dotY != 0 )
				h = (float) ( pImage->height() / dotY );
			d = m_fDepth;
		}

		QString str;

		str.sprintf( "%lf", w );
		IDC_EDIT_WIDTH->setText( str );
		str.sprintf( "%lf", h );
		IDC_EDIT_HEIGHT->setText( str );
		str.sprintf( "%lf", d );
		IDC_EDIT_DEPTH->setText( str );

		m_fWidth = w;
		m_fHeight = h;
		m_fDepth = d;
	}
	else
	{
		QString str;

		str.sprintf( "%lf", m_fWidth );
		IDC_EDIT_WIDTH->setText( str );
		str.sprintf( "%lf", m_fHeight );
		IDC_EDIT_HEIGHT->setText( str );
		str.sprintf( "%lf", m_fDepth );
		IDC_EDIT_DEPTH->setText( str );
	}

	slotValueChanged();

	IDC_LBL_DESCRIPTION->setText(
		tr(
"Please specify the size of your voxel data."
		) );
} /* init */

void CImportVoxelInfoDlg::slotValueChanged()
{
	QString str;
	double w, h, d;

	str = IDC_EDIT_WIDTH->text();
	w = str.toDouble();
	m_fWidth = w;

	str = IDC_EDIT_HEIGHT->text();
	h = str.toDouble();
	m_fHeight = h;

	str = IDC_EDIT_DEPTH->text();
	d = str.toDouble();
	m_fDepth = d;

	if ( w == 0.0 || h == 0.0 || d == 0.0 )
		m_pParent->nextButton()->setEnabled( false );
	else
		m_pParent->nextButton()->setEnabled( true );
} /* slotValueChanged */


////////////////////////////////////////////////////////////////////////////
//
// CImportROIDlg
//

CImportROIDlg::CImportROIDlg( QWidget *parent, const char *name, bool modal, WFlags fl )
	: CImportROIDlg_base( parent, name, modal, fl )
{
	m_pParent = (CImportStackWizard *) parent;
	ASSERT( m_pParent );

	m_nWidthUp = -1;
	m_nWidthDown = -1;

	m_nHeightUp = -1;
	m_nHeightDown = -1;

	m_nDepthUp = -1;
	m_nDepthDown = -1;

	connect( IDC_BTN_DEFAULT, SIGNAL( clicked() ),
		this, SLOT( slotSetDefault() ) );

	connect( IDC_BTN_SELECT, SIGNAL( clicked() ),
		this, SLOT( slotSelectROI() ) );

	connect( IDC_SPIN_WIDTH_DOWN, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotWidthDown( int ) ) );
	connect( IDC_SPIN_WIDTH_UP, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotWidthUp( int ) ) );
	
	connect( IDC_SPIN_HEIGHT_DOWN, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotHeightDown( int ) ) );
	connect( IDC_SPIN_HEIGHT_UP, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotHeightUp( int ) ) );
	
	connect( IDC_SPIN_DEPTH_DOWN, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotDepthDown( int ) ) );
	connect( IDC_SPIN_DEPTH_UP, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotDepthUp( int ) ) );
} /* CImportROIDlg */

CImportROIDlg::~CImportROIDlg()
{
} /* ~CImportROIDlg */

void CImportROIDlg::init()
{
	if ( m_nWidthUp == -1 && m_nHeightUp == -1 && m_nDepthUp == -1 )
		slotSetDefault();
	else
	{
		IDC_SPIN_WIDTH_DOWN->setValue( m_nWidthDown );
		IDC_SPIN_WIDTH_UP->setValue( m_nWidthUp );

		IDC_SPIN_HEIGHT_DOWN->setValue( m_nHeightDown );
		IDC_SPIN_HEIGHT_UP->setValue( m_nHeightUp );

		IDC_SPIN_DEPTH_DOWN->setValue( m_nDepthDown );
		IDC_SPIN_DEPTH_UP->setValue( m_nDepthUp );
	}

	setRequiredMemory();

	IDC_LBL_DESCRIPTION->setText(
		tr(
			"Please choose your Region of Interest(ROI).\n"
#ifdef QT230
			"\n"
#endif
			"You can select it graphically using mouse drag."
		) );
} /* init */

// private
void CImportROIDlg::setRequiredMemory()
{
	int w = m_nWidthUp - m_nWidthDown + 1;
	int h = m_nHeightUp - m_nHeightDown + 1;
	int d = m_nDepthUp - m_nDepthDown + 1;
	int memory = w * h * d * VOXEL_MEMORY_SIZE;		//XXX
	QString str;

	// change label color
	QPalette p = IDC_LBL_MEMORY->palette();
	p.setColor( QColorGroup::Foreground, QColor( 0, 0, 255 ) );		// blue
	IDC_LBL_MEMORY->setPalette( p );

	// set label
	str.sprintf( "%d Byte(s)", memory );
	IDC_LBL_MEMORY->setText( str );
} /* setRequiredMemory */

void CImportROIDlg::setWidthRange( int up, int down )
{
	m_nWidthUp = up;
	m_nWidthDown = down;

	setRequiredMemory();
} /* setWidthRange */

void CImportROIDlg::setHeightRange( int up, int down )
{
	m_nHeightUp = up;
	m_nHeightDown = down;

	setRequiredMemory();
} /* setHeightRange */

void CImportROIDlg::setDepthRange( int up, int down )
{
	m_nDepthUp = up;
	m_nDepthDown = down;

	setRequiredMemory();
} /* setDepthRange */

// slot
void CImportROIDlg::slotSetDefault()
{
	int w1, w2;
	int h1, h2;
	int d1, d2;

	w1 = h1 = d1 = 1;

	QImage *pImage = m_pParent->image();
	if ( pImage == NULL )
	{
		/*
		QMessageBox::information( this, "File Error",
			"Error of Image File" );
		*/

		w2 = h2 = d2 = 1;

		m_pParent->nextButton()->setEnabled( false );			
	}
	else
	{
		QImage *pImage = m_pParent->image();

		w2 = pImage->width();
		h2 = pImage->height();
		d2 = m_pParent->numSlice();
	}

	IDC_SPIN_WIDTH_DOWN->setMinValue( 1 );
	IDC_SPIN_WIDTH_DOWN->setMaxValue( w2 );
	IDC_SPIN_WIDTH_DOWN->setValue( 1 );
	IDC_SPIN_WIDTH_UP->setMinValue( 1 );
	IDC_SPIN_WIDTH_UP->setMaxValue( w2 );
	IDC_SPIN_WIDTH_UP->setValue( w2 );

	IDC_SPIN_HEIGHT_DOWN->setMinValue( 1 );
	IDC_SPIN_HEIGHT_DOWN->setMaxValue( w2 );
	IDC_SPIN_HEIGHT_DOWN->setValue( 1 );
	IDC_SPIN_HEIGHT_UP->setMinValue( 1 );
	IDC_SPIN_HEIGHT_UP->setMaxValue( h2 );
	IDC_SPIN_HEIGHT_UP->setValue( h2 );

	IDC_SPIN_DEPTH_DOWN->setMinValue( 1 );
	IDC_SPIN_DEPTH_DOWN->setMaxValue( d2 );
	IDC_SPIN_DEPTH_DOWN->setValue( 1 );
	IDC_SPIN_DEPTH_UP->setMinValue( 1 );
	IDC_SPIN_DEPTH_UP->setMaxValue( d2 );
	IDC_SPIN_DEPTH_UP->setValue( d2 );
		
	m_nWidthUp = w2;
	m_nWidthDown = w1;

	m_nHeightUp = h2;
	m_nHeightDown = h1;

	m_nDepthUp = d2;
	m_nDepthDown = d1;

	setRequiredMemory();
} /* slotSetDefault */

// slot
void CImportROIDlg::slotSelectROI()
{
	WFlags f = Qt::WType_TopLevel | Qt::WStyle_MinMax;
#ifdef Q_WS_WIN
	f |= Qt::WStyle_SysMenu;
#endif

	CImportROIPreviewDlg *preview
		= new CImportROIPreviewDlg( m_pParent, this, "preview", true, f );

	if ( preview->exec() != QDialog::Accepted )
		return;
} /* slotSelectROI */

// slot
void CImportROIDlg::slotWidthDown( int value )
{
	m_nWidthDown = value;

	setRequiredMemory();
} /* slotWidthDown */

// slot
void CImportROIDlg::slotWidthUp( int value )
{
	m_nWidthUp = value;

	setRequiredMemory();
} /* slotWidthUp */

// slot
void CImportROIDlg::slotHeightDown( int value )
{
	m_nHeightDown = value;

	setRequiredMemory();
} /* slotHeightDown */

// slot
void CImportROIDlg::slotHeightUp( int value )
{
	m_nHeightUp = value;

	setRequiredMemory();
} /* slotHeightUp */

// slot
void CImportROIDlg::slotDepthDown( int value )
{
	m_nDepthDown = value;

	setRequiredMemory();
} /* slotDepthDown */

// slot
void CImportROIDlg::slotDepthUp( int value )
{
	m_nDepthUp = value;

	setRequiredMemory();
} /* slotDepthUp */


////////////////////////////////////////////////////////////////////////////
//
// CImportROIPreviewDlg
//

CImportROIPreviewDlg::CImportROIPreviewDlg( CImportStackWizard *pWizard,
				QWidget *parent, const char *name, bool modal, WFlags fl )
	: CImportROIPreviewDlg_base( parent, name, modal, fl )
{
	m_pWizard = pWizard;
	ASSERT( m_pWizard );
	m_pParent = (CImportROIDlg *) parent;
	ASSERT( m_pParent );

	m_pScrollView = NULL;
	m_pROIPreviewArea = NULL;

	connect( this, SIGNAL( viewResized() ),
		this, SLOT( slotMoveImageToCenter() ) );

	init();
} /* CImportROIPreviewDlg */

CImportROIPreviewDlg::~CImportROIPreviewDlg()
{
	if ( m_pScrollView )
	{
		delete m_pScrollView;
		m_pScrollView = NULL;
	}
} /* ~CImportROIPreviewDlg */

void CImportROIPreviewDlg::init()
{
	// 1. set scrollview
	m_pScrollView = new QScrollView( this );
	ASSERT( m_pScrollView );
	m_pScrollView->setGeometry( 0, 0, width(), height()-60 );

	// 2. set scroll area
	m_pROIPreviewArea = new CROIPreviewArea( this );
	ASSERT( m_pROIPreviewArea );
	m_StringList = m_pWizard->fileList();
	m_ImageFormat= m_pWizard->imageFormat();
	QStringList::Iterator it = m_StringList.begin();
	m_pROIPreviewArea->init( *it, m_ImageFormat );

	m_pScrollView->addChild( m_pROIPreviewArea );

	// 3. set bottom frame
	IDC_FRAME_ROI->setGeometry( 0, height()-60, width(), 60 );
	IDC_SPIN_SLICE->setRange( 1, m_StringList.count() );
	IDC_SPIN_SLICE->setValue( 1 );

	// 4. signal-slot
	connect( IDC_SPIN_SLICE, SIGNAL( valueChanged( int) ),
		this, SLOT( slotSliceChanged( int ) ) );
	connect( IDC_BTN_ENDROI, SIGNAL( clicked() ),
		this, SLOT( slotEndROI() ) );
} /* init */

// slot
void CImportROIPreviewDlg::slotSliceChanged( int slice )
{
	m_pROIPreviewArea->init( m_StringList[slice-1], m_ImageFormat );
	m_pROIPreviewArea->repaint( false );
} /* slotSliceChanged */

// slot
void CImportROIPreviewDlg::slotEndROI()
{
	QString str;
	int x1, y1;
	int x2, y2;

	str = IDC_LBL_X->text();
	x1 = str.toInt();
	m_pParent->IDC_SPIN_WIDTH_DOWN->setValue( x1 );		// width down
	str = IDC_LBL_Y->text();
	y1 = str.toInt();
	m_pParent->IDC_SPIN_HEIGHT_DOWN->setValue( y1 );	// height down

	str = IDC_LBL_WIDTH->text();
	x2 = x1 + str.toInt();
	m_pParent->IDC_SPIN_WIDTH_UP->setValue( x2 );		// width up
	str = IDC_LBL_HEIGHT->text();
	y2 = y1 + str.toInt();
	m_pParent->IDC_SPIN_HEIGHT_UP->setValue( y2 );		// height up

	// end dialog
	QDialog::accept();
} /* slotEndROI */

// slot
void CImportROIPreviewDlg::slotMoveImageToCenter()
{
	int x = 0;
	int y = 0;
	int w = m_pScrollView->width();
	int h = m_pScrollView->height();

	if ( m_pROIPreviewArea->width() < w )
		x = (int) ( w - m_pROIPreviewArea->width() ) / 2;

	if ( m_pROIPreviewArea->height() < h )
		y = (int) ( h - m_pROIPreviewArea->height() ) / 2;

	m_pScrollView->moveChild( m_pROIPreviewArea, x, y );
} /* slotMoveImageToCenter */

// virtual
void CImportROIPreviewDlg::resizeEvent( QResizeEvent * /*e*/ )
{
	m_pScrollView->setGeometry( 0, 0, width(), height()-60 );
	IDC_FRAME_ROI->setGeometry( 0, height()-60, width(), 60 );

	emit viewResized();
} /* resizeEvent */


////////////////////////////////////////////////////////////////////////////
//
// CROIPreviewArea
//

CROIPreviewArea::CROIPreviewArea( QWidget *parent, const char *name, WFlags fl )
	: QWidget( parent, name, fl )
{
	m_pParent = (CImportROIPreviewDlg *) parent;
	ASSERT( m_pParent );

	m_pBuffer = NULL;

	setBackgroundMode( NoBackground );
} /* CROIPreviewArea */

CROIPreviewArea::~CROIPreviewArea()
{
	if ( m_pBuffer )
	{
		delete m_pBuffer;
		m_pBuffer = NULL;
	}
} /* ~CROIPreviewArea */

bool CROIPreviewArea::init( QString fileName, QString imageFormat )
{
	if ( m_pBuffer ) 
	{
		delete m_pBuffer;
		m_pBuffer = NULL;
	}

	m_pBuffer = new QImage;

	if ( imageFormat == "DICOM" )
	{
		CDicomFile df;
		if ( !df.init( fileName ) )
			return false;

		WORD w = df.width();
		WORD h = df.height();

		*m_pBuffer = df.image().copy( 0, 0, w, h );

		df.close();
	}
	else
		m_pBuffer->load( fileName, imageFormat );

	m_szImage.setWidth( m_pBuffer->width() );
	m_szImage.setHeight( m_pBuffer->height() );

	m_bChangeFlag = true;

	m_pParent->IDC_LBL_X->setText( "1" );
	m_pParent->IDC_LBL_Y->setText( "1" );
	QString str;
	str.sprintf( "%d", m_szImage.width() );
	m_pParent->IDC_LBL_WIDTH->setText( str );
	str.sprintf( "%d", m_szImage.height() );
	m_pParent->IDC_LBL_HEIGHT->setText( str );

	resize( m_szImage );

	return true;
} /* init */

// virtual
void CROIPreviewArea::paintEvent( QPaintEvent * /*e*/ )
{	
	bitBlt( this, 0, 0, m_pBuffer );
} /* paintEvent */

// virtual
void CROIPreviewArea::resizeEvent( QResizeEvent * /*e*/ )
{
	resize( m_szImage );
} /* resizeEvent */

// virtual
void CROIPreviewArea::mousePressEvent( QMouseEvent *e )
{
	// print mouse position (x, y) to frame
	QString str;
	str.sprintf( "%d", e->pos().x() );		// x
	m_pParent->IDC_LBL_X->setText( str );
	str.sprintf( "%d", e->pos().y() );		// y
	m_pParent->IDC_LBL_Y->setText( str );

	// remove current selection region
	if ( m_bChangeFlag == false )
	{
		QRect rt1( m_ptFirst.x(), m_ptFirst.y(),
			m_ptLast.x()-m_ptFirst.x(), m_ptLast.y()-m_ptFirst.y() );

		QPen pen( QColor( 0, 0, 255 ), 2 );
		QPainter p;
		p.begin( this );
			p.setPen( pen );
			p.setRasterOp( NotXorROP );
			p.drawRect( rt1 );
		p.end();
	}
	else
		m_bChangeFlag = false;

	// set new position
	m_ptFirst = m_ptLast = e->pos();
} /* mousePressEvent */

// virtual
void CROIPreviewArea::mouseMoveEvent( QMouseEvent *e )
{
	int w1 = m_ptLast.x() - m_ptFirst.x();
	int h1 = m_ptLast.y() - m_ptFirst.y();
	int w2 = e->x() - m_ptFirst.x();
	int h2 = e->y() - m_ptFirst.y();

	// print width & height of selected region
	QString str;
	str.sprintf( "%d", w2 );					// width
	m_pParent->IDC_LBL_WIDTH->setText( str );
	str.sprintf( "%d", h2 );					// height
	m_pParent->IDC_LBL_HEIGHT->setText( str );

	// draw ROI rectangle
	QRect rt1( m_ptFirst.x(), m_ptFirst.y(), w1, h1 );
	QRect rt2( m_ptFirst.x(), m_ptFirst.y(), w2, h2 );

	QPen pen( QColor( 0, 0, 255 ), 2 );
	QPainter p;
		p.begin( this );
		p.setPen( pen );
		p.setRasterOp( NotXorROP );
		p.drawRect( rt1 );
		p.drawRect( rt2 );
	p.end();

	m_ptLast = e->pos();
} /* mouseMoveEvent */

// virtual
void CROIPreviewArea::mouseReleaseEvent( QMouseEvent *e )
{
//	int w1 = m_ptLast.x() - m_ptFirst.x();
//	int h1 = m_ptLast.y() - m_ptFirst.y();
	int w2 = e->x() - m_ptFirst.x();
	int h2 = e->y() - m_ptFirst.y();

	if ( w2 < 0 )
	{
		QString str;

		str.sprintf( "%d", m_ptFirst.x() + w2 );
		m_pParent->IDC_LBL_X->setText( str );
		str.sprintf( "%d", -w2 );
		m_pParent->IDC_LBL_WIDTH->setText( str );
	}

	if ( h2 < 0 )
	{
		QString str;

		str.sprintf( "%d", m_ptFirst.y() + h2 );
		m_pParent->IDC_LBL_Y->setText( str );
		str.sprintf( "%d", -h2 );
		m_pParent->IDC_LBL_HEIGHT->setText( str );
	}
} /* mouseReleaseEvent */


////////////////////////////////////////////////////////////////////////////
//
// CImportBoneColorDlg
//

CImportBoneColorDlg::CImportBoneColorDlg( QWidget *parent, const char *name, bool modal, WFlags fl )
	: CImportBoneColorDlg_base( parent, name, modal, fl )
{
	m_bWhiteIsBone = true;

	connect( IDC_RADIO_WHITE, SIGNAL( clicked() ),
		this, SLOT( slotWhiteButtonClicked() ) );
	connect( IDC_RADIO_BLACK, SIGNAL( clicked() ),
		this, SLOT( slotBlackButtonClicked() ) );
} /* CImportBoneColorDlg */

CImportBoneColorDlg::~CImportBoneColorDlg()
{
} /* ~CImportBoneColorDlg */

void CImportBoneColorDlg::init()
{
	if ( m_bWhiteIsBone )
	{
		IDC_RADIO_WHITE->setChecked( true );
		IDC_RADIO_BLACK->setChecked( false );
	}
	else
	{
		IDC_RADIO_WHITE->setChecked( false );
		IDC_RADIO_BLACK->setChecked( true );
	}

	IDC_LBL_DESCRIPTION->setText(
		tr(
"Please choose bone voxel color."
		) );
} /* init */

// slot
void CImportBoneColorDlg::slotWhiteButtonClicked()
{
	m_bWhiteIsBone = true;
} /* slotWhiteBoneClicked */

// slot
void CImportBoneColorDlg::slotBlackButtonClicked()
{
	m_bWhiteIsBone = false;
} /* slotBlackBoneClicked */