/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "MDIWindow.h"
#include "RenderMDIWindow.h"
#include "VolumeFile.h"
#include "RenderView.h"
#include "FEModelSaveAsImageDlg.h"
#include "RenderedImageSizeDlg.h"
#include "RenderInfoDlg.h"

#include <qimage.h>
#include <qpixmap.h>
#include <qfile.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qmessagebox.h>

#include "volume.h"

// debug
#include "MainWindow.h"
#include <qstatusbar.h>


///////////////////////////////////////////////////////////////////////////
//
// RenderMDIWindow
//

RenderMDIWindow::RenderMDIWindow( QWidget *parent, const char *name, WFlags f )
	: QMainWindow( parent, name, f )
{
	m_pVolume = NULL;
	m_pMDIWindow = NULL;
	m_vpc = NULL;

	m_fViewport = 1.0;

	m_bRotateX = true;
	m_bRotateY = true;
	m_bRotateZ = true;

	m_nImageSizeRatio = 4;	// 4*128 = 512;
	m_pRenderedImage = NULL;
	m_pRawVolume = NULL;

	m_pRenderView = new CRenderView( this );
	ASSERT( m_pRenderView );

	setFocusProxy( m_pRenderView );
	setCentralWidget( m_pRenderView );

//	setBackgroundMode( NoBackground );
} /* RenderMDIWindow */

RenderMDIWindow::~RenderMDIWindow()
{
	if ( m_pMDIWindow != NULL )
		m_pMDIWindow->unsetRenderView();

	if ( m_pRenderView != NULL )
	{
		delete m_pRenderView;
		m_pRenderView = NULL;
	}

	if ( m_vpc != NULL )
		vpDestroyContext( m_vpc );

	if ( m_pRawVolume != NULL )
	{
		delete m_pRawVolume;
		m_pRawVolume = NULL;
	}

	if ( m_pRenderedImage != NULL )
	{
		delete m_pRenderedImage;
		m_pRenderedImage = NULL;
	}
} /* ~RenderMDIWindow */

bool RenderMDIWindow::init( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );

	if ( !makeRawVolume() )
	{
		g_pMainWindow->statusBar()->message( "Ready" );
		return false;
	}

	if ( !renderVolume() )
	{
		g_pMainWindow->statusBar()->message( "Ready" );
		return false;
	}

	g_pMainWindow->statusBar()->message( "Ready" );

	m_pRenderView->init( this );

	QString fileName = m_pVolume->volumeFileName();
	if ( fileName.isEmpty() )
		fileName = "unnamed Render View";
		
	setCaption( tr( "Render View : " ) + fileName );

	resize( 512, 512 );

	return true;
} /* init */ 

CVolume *RenderMDIWindow::volume()
{
	return m_pVolume;
} /* volume */

void RenderMDIWindow::setMDIWindow( MDIWindow *pMDIWindow )
{
	m_pMDIWindow = pMDIWindow;
	ASSERT( m_pMDIWindow );
} /* setMDIWindow */

MDIWindow *RenderMDIWindow::mdiWindow()
{
	if ( m_pMDIWindow )
		return m_pMDIWindow;
	else
		return NULL;
} /* mdiWindow */

bool RenderMDIWindow::setRotateX( float x )
{
	m_bRotateX = true;

	m_ri.rotate_angle[0] = x;

	if ( renderScene() )
		return true;
	else
	{
		m_bRotateX = false;
		return false;
	}

	return true;
} /* setRotateX */

bool RenderMDIWindow::setRotateY( float y )
{
	m_bRotateY = true;

	m_ri.rotate_angle[1] = y;

	if ( renderScene() )
		return true;
	else
	{
		m_bRotateY = false;
		return false;
	}
} /* setRotateY */

bool RenderMDIWindow::setRotateZ( float z )
{
	m_bRotateZ = true;

	m_ri.rotate_angle[2] = z;

	if ( renderScene() )
		return true;
	else
	{
		m_bRotateZ = false;
		return false;
	}
} /* setRotateZ */

bool RenderMDIWindow::zoomIn()
{
	m_fViewport -= 0.1f;

	m_ri.view_port[0] = -m_fViewport;
	m_ri.view_port[1] = m_fViewport;

	m_ri.view_port[2] = -m_fViewport;
	m_ri.view_port[3] = m_fViewport;

	m_ri.view_port[4] = -m_fViewport;
	m_ri.view_port[5] = m_fViewport;

	if ( renderScene() )
		return true;
	else
		return false;
} /* zoomIn */

bool RenderMDIWindow::zoomOut()
{
	m_fViewport += 0.1f;

	m_ri.view_port[0] = -m_fViewport;
	m_ri.view_port[1] = m_fViewport;

	m_ri.view_port[2] = -m_fViewport;
	m_ri.view_port[3] = m_fViewport;

	m_ri.view_port[4] = -m_fViewport;
	m_ri.view_port[5] = m_fViewport;

	if ( renderScene() )
		return true;
	else
		return false;
} /* zoomOut */

bool RenderMDIWindow::plusX()
{
	m_bRotateX = true;

	m_ri.rotate_angle[0] = 5.0;

	if ( renderScene() )
		return true;
	else
	{
		m_bRotateX = false;
		return false;
	}
} /* plusX */

bool RenderMDIWindow::minusX()
{
	m_bRotateX = true;

	m_ri.rotate_angle[0] = -5.0;

	if ( renderScene() )
		return true;
	else
	{
		m_bRotateX = false;
		return false;
	}
} /* minusX */

bool RenderMDIWindow::plusY()
{
	m_bRotateY = true;

	m_ri.rotate_angle[1] = -5.0;

	if ( renderScene() )
		return true;
	else
	{
		m_bRotateY = false;
		return false;
	}
} /* plusY */

bool RenderMDIWindow::minusY()
{
	m_bRotateY = true;

	m_ri.rotate_angle[1] = 5.0;

	if ( renderScene() )
		return true;
	else
	{
		m_bRotateY = false;
		return false;
	}
} /* minusY */

bool RenderMDIWindow::plusZ()
{
	m_bRotateZ = true;

	m_ri.rotate_angle[2] = 5.0;

	if ( renderScene() )
		return true;
	else
	{
		m_bRotateZ = false;
		return false;
	}
} /* plusZ */

bool RenderMDIWindow::minusZ()
{
	m_bRotateZ = true;

	m_ri.rotate_angle[2] = -5.0;

	if ( renderScene() )
		return true;
	else
	{
		m_bRotateZ = false;
		return false;
	}
} /* minusZ */

bool RenderMDIWindow::renderScene()
{
	if ( !renderVolume() )
		return false;

	m_pRenderView->renderViewArea()->init( m_pRenderedImage, m_nImageSizeRatio );
	m_pRenderView->renderViewArea()->repaint( false );
	m_pRenderView->slotMoveImageToCenter();

	return true;
} /* renderScene */


/*************/
/* rendering */
/*************/
bool RenderMDIWindow::makeRawVolume()
{
	g_pMainWindow->statusBar()->message( "Make Raw Volume..." );


	// declare variables
    unsigned density_size;		// size of density data
    unsigned raw_volume_size;	// size of volume
	short xlen, ylen, zlen;		// lengths of volume
	BYTE *density;

	// get the lengths of the volume
	ASSERT( m_pVolume );
	xlen = m_pVolume->volumeWidth();
	ylen = m_pVolume->volumeHeight();
	zlen = m_pVolume->volumeDepth();
    
	// create a context
	if ( m_vpc )
		vpDestroyContext( m_vpc );

    m_vpc = vpCreateContext();
	if ( !m_vpc )
	{
		QMessageBox::critical( g_pMainWindow, "Creation Error",
			"Can't create volpack context..." );

		return false;
	}

    // describe the layout of the volume
    vpSetVolumeSize( m_vpc, xlen, ylen, zlen );

    vpSetVoxelSize( m_vpc, BYTES_PER_VOXEL, VOXEL_FIELDS, SHADE_FIELDS, CLSFY_FIELDS );
    vpSetVoxelField( m_vpc, NORMAL_FIELD, NORMAL_SIZE, NORMAL_OFFSET, NORMAL_MAX );
    vpSetVoxelField( m_vpc, DENSITY_FIELD, DENSITY_SIZE, DENSITY_OFFSET, DENSITY_MAX );
    vpSetVoxelField( m_vpc, GRADIENT_FIELD, GRADIENT_SIZE, GRADIENT_OFFSET, GRADIENT_MAX );

    // allocate space for the raw data and the volume
	density_size = xlen * ylen * zlen;
    raw_volume_size = density_size * BYTES_PER_VOXEL;

	if ( m_pRawVolume != NULL )
	{
		delete m_pRawVolume;
		m_pRawVolume = NULL;
	}

	m_pRawVolume = new char[raw_volume_size];
	if ( !m_pRawVolume )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Can't allocate memory for raw volume..." );

		return false;
	}

    vpSetRawVoxels( m_vpc, (char *) m_pRawVolume, raw_volume_size, BYTES_PER_VOXEL,
		   xlen * BYTES_PER_VOXEL, xlen * ylen * BYTES_PER_VOXEL );

	// generate voxel density
	density = new BYTE[density_size];
	if ( !density )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Can't allocate memory for density..." );

		return false;
	}

	for ( DWORD i=0 ; i<m_pVolume->totalNumVoxel() ; i++ )
		density[i] = m_pVolume->voxelColor( i );

    // compute surface normals (for shading) and
    // gradient magnitudes (for classification)
	if ( vpVolumeNormals( m_vpc, density, density_size, DENSITY_FIELD,
			GRADIENT_FIELD, NORMAL_FIELD) != VP_OK )
	{
		QMessageBox::critical( g_pMainWindow, "Raw Volume Error",
			"Can't make volume normals..." );

		return false;
	}

    // compute the octree
	if ( vpSetClassifierTable( m_vpc, DENSITY_PARAM, DENSITY_FIELD, NULL, 0 ) != VP_OK )
	{
		QMessageBox::critical( g_pMainWindow, "Octree Error",
			"Can't set density parameters..." );

		return false;
	}

	if ( vpSetClassifierTable( m_vpc, GRADIENT_PARAM, GRADIENT_FIELD, NULL, 0 ) != VP_OK )
	{
		QMessageBox::critical( g_pMainWindow, "Octree Error",
			"Can't set gradient parameters..." );

		return false;
	}

    vpMinMaxOctreeThreshold( m_vpc, DENSITY_PARAM, OCTREE_DENSITY_THRESH );
    vpMinMaxOctreeThreshold( m_vpc, GRADIENT_PARAM, OCTREE_GRADIENT_THRESH );

    if ( vpCreateMinMaxOctree( m_vpc, 0, OCTREE_BASE_NODE_SIZE ) != VP_OK )
	{
		QMessageBox::critical( g_pMainWindow, "Octree Error",
			"Can't make min-max octree..." );

		return false;
	}

	// load rendering information
	setDefaultRenderInfo();

    return true;
} /* makeRawVolume */

bool RenderMDIWindow::renderVolume()
{
	g_pMainWindow->statusBar()->message( "Render Raw Volume..." );


	if ( !m_vpc )
		return false;

	// declare local variables
	float density_ramp[DENSITY_MAX+1];		// opacity as a function of density
	float gradient_ramp[GRADIENT_MAX+1];	// opacity as a function of gradient magnitude
	float shade_table[NORMAL_MAX+1];		// shading lookup table
	short image_width, image_height;		// rendered image width & height
	int image_size;							// size of rendered image

	// calculate image size
	image_width = m_nImageSizeRatio * 128;
	image_height = image_width;
	image_size = image_width * image_height;

	if ( m_pRenderedImage != NULL )
	{
		delete m_pRenderedImage;
		m_pRenderedImage = NULL;
	}

	m_pRenderedImage = new BYTE[image_size];
	if ( !m_pRenderedImage )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Can't allocate memory for rendered image..." );

		return false;
	}

	// set the classification function
	vpRamp( density_ramp, sizeof(float), DENSITY_RAMP_POINTS, DensityRampX, DensityRampY );
	vpSetClassifierTable(
			m_vpc, DENSITY_PARAM, DENSITY_FIELD, density_ramp, sizeof(density_ramp) );

	vpRamp( gradient_ramp, sizeof(float), GRADIENT_RAMP_POINTS, GradientRampX, GradientRampY );
	vpSetClassifierTable(
			m_vpc, GRADIENT_PARAM, GRADIENT_FIELD, gradient_ramp, sizeof(gradient_ramp) );

	// set the shading parameters
	vpSetLookupShader( m_vpc, 1, 1, NORMAL_FIELD, shade_table, sizeof(shade_table), 0, NULL, 0 );

	vpSetd( m_vpc, VP_MIN_VOXEL_OPACITY, m_ri.voxel_opacity[0] );
    vpSetd( m_vpc, VP_MAX_RAY_OPACITY, m_ri.voxel_opacity[1] );
	    
	vpSetMaterial( m_vpc, VP_MATERIAL0, VP_AMBIENT, VP_BOTH_SIDES,
			m_ri.mat_ambient[0], m_ri.mat_ambient[1], m_ri.mat_ambient[2] );
    vpSetMaterial( m_vpc, VP_MATERIAL0, VP_DIFFUSE, VP_BOTH_SIDES,
			m_ri.mat_diffuse[0], m_ri.mat_diffuse[1], m_ri.mat_diffuse[2] );
    vpSetMaterial( m_vpc, VP_MATERIAL0, VP_SPECULAR, VP_BOTH_SIDES,
			m_ri.mat_specular[0], m_ri.mat_specular[1], m_ri.mat_specular[2] );
    vpSetMaterial( m_vpc, VP_MATERIAL0, VP_SHINYNESS, VP_BOTH_SIDES,
			m_ri.mat_shinyness, 0.0, 0.0 );
    
	vpSetLight( m_vpc, VP_LIGHT0, VP_DIRECTION,
			m_ri.light_direction[0], m_ri.light_direction[1], m_ri.light_direction[2] );
    vpSetLight( m_vpc, VP_LIGHT0, VP_COLOR,
			m_ri.light_color[0], m_ri.light_color[1], m_ri.light_color[2] );
    vpEnable( m_vpc, VP_LIGHT0, 1 );

    vpSetDepthCueing( m_vpc, m_ri.depth_cueing[0], m_ri.depth_cueing[1] );
    vpEnable( m_vpc, VP_DEPTH_CUE, 1 );

    // set the initial viewing parameters
    vpSeti( m_vpc, VP_CONCAT_MODE, VP_CONCAT_LEFT );

	if ( m_bRotateX )
	{
		vpRotate( m_vpc, VP_X_AXIS, m_ri.rotate_angle[0] );
		m_bRotateX = false;
	}
	if ( m_bRotateY )
	{
		vpRotate( m_vpc, VP_Y_AXIS, m_ri.rotate_angle[1] );
		m_bRotateY = false;
	}
	if ( m_bRotateZ )
	{
		vpRotate( m_vpc, VP_Z_AXIS, m_ri.rotate_angle[2] );
		m_bRotateZ = false;
	}

    vpCurrentMatrix( m_vpc, VP_PROJECT );
    vpIdentityMatrix( m_vpc );
	
	vpWindow( m_vpc, VP_PARALLEL,
			m_ri.view_port[0], m_ri.view_port[1],
			m_ri.view_port[2], m_ri.view_port[3],
			m_ri.view_port[4], m_ri.view_port[5] );
		    
	vpCurrentMatrix( m_vpc, VP_MODEL );

    // set the image buffer
    vpSetImage(	m_vpc, 
		(BYTE *) m_pRenderedImage, image_width, image_height, image_width, VP_LUMINANCE );


	/*******************************/
    /* render and store the images */
	/*******************************/

	// compute shading lookup table
	if ( vpShadeTable( m_vpc ) != VP_OK )
		return false;

	// rendering the raw volume with octree
	// printf( " Rendering the Raw Volume with Octree...\n\n" );

	if ( vpRenderRawVolume( m_vpc ) != VP_OK )
	{
		QMessageBox::critical( g_pMainWindow, "Render Error",
				"Can't render raw volume..." );

		return false;
	}

	return true;
} /* renderVolume */

BYTE *RenderMDIWindow::renderedImage()
{
	if ( m_pRenderedImage )
		return m_pRenderedImage;
	else
		return NULL;
} /* renderedImage */


// set default rendering information
void RenderMDIWindow::setDefaultRenderInfo()
{
	m_ri.voxel_opacity[0] = 0.05;
	m_ri.voxel_opacity[1] = 0.95;

	m_ri.mat_ambient[0] = 0.18;
	m_ri.mat_ambient[1] = 0.18;
	m_ri.mat_ambient[2] = 0.18;

	m_ri.mat_diffuse[0] = 0.35;
	m_ri.mat_diffuse[1] = 0.35;
	m_ri.mat_diffuse[2] = 0.35;

	m_ri.mat_specular[0] = 0.39;
	m_ri.mat_specular[1] = 0.39;
	m_ri.mat_specular[2] = 0.39;

	m_ri.mat_shinyness = 10.0;

	m_ri.light_direction[0] = 0.3;
	m_ri.light_direction[1] = 0.3;
	m_ri.light_direction[2] = 1.0;

	m_ri.light_color[0] = 1.0;
	m_ri.light_color[1] = 1.0;
	m_ri.light_color[2] = 1.0;

	m_ri.depth_cueing[0] = 1.4;
	m_ri.depth_cueing[1] = 1.5;

	m_ri.rotate_angle[0] = 0.;
	m_ri.rotate_angle[1] = 0.;
	m_ri.rotate_angle[2] = 0.;

	m_ri.view_port[0] = -1.0;
	m_ri.view_port[1] =  1.0;

	m_ri.view_port[2] = -1.0;
	m_ri.view_port[3] =  1.0;

	m_ri.view_port[4] = -1.0;
	m_ri.view_port[5] =  1.0;
} /* setDefaultRenderInfo */

void RenderMDIWindow::deleteVPContext()
{
	if ( m_vpc != NULL )
	{
		vpDestroyContext( m_vpc );
		m_vpc = NULL;
	}
} /* deleteVPContext */

// slot
bool RenderMDIWindow::slotSaveAsImage()
{
	CFEModelSaveAsImageDlg dlg( this, NULL, true );
	if ( dlg.init( false ) )
	{
		if ( dlg.exec() != QDialog::Accepted )
			return false;

		// check file existance
		QString strFileName = dlg.IDC_EDIT_FILENAME->text();
		QString strFileType = dlg.IDC_COMBO_FILEFORMAT->currentText();
		QString strMsg = strFileName
			+ "This file already exists.\n"
			+ "Replace existing file ?";

		if ( QFile::exists( strFileName ) )
			QFile::remove( strFileName );

		QImage image = m_pRenderView->imagePixmap().convertToImage();

		if ( !image.save( strFileName, strFileType ) )
			return false;

		QMessageBox::information( this, "Save as Image",
			tr( "Saving as Image Done..." ) );
	}

	return true;
} /* slotSaveAsImage */

// slot
bool RenderMDIWindow::slotRenderedImageSize()
{
	CRenderedImageSizeDlg dlg( this, NULL, true );
	dlg.init( m_nImageSizeRatio );
	if ( dlg.exec() != QDialog::Accepted )
		return false;

	if ( m_nImageSizeRatio == dlg.renderedImageSizeRatio() )
		return false;

	m_nImageSizeRatio = dlg.renderedImageSizeRatio();

	renderScene();

	return true;
} /* slotRenderedImageSize */

// slot
bool RenderMDIWindow::slotRenderInfo()
{
	CRenderInfoDlg dlg( this, NULL, true );
	dlg.init( m_ri );
	if ( dlg.exec() != QDialog::Accepted )
		return false;

	m_ri = dlg.renderInfo();

	renderScene();

	return true;
} /* slotRenderInfo */
