/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#ifndef _DEFINE_H_
#define _DEFINE_H_

#include "global_define.h"
#include <qstringlist.h>


/***********/
/* defines */
/***********/

#ifndef PI
#define PI 3.141592
#endif


// volume signature is 0x4D56 : 'VM' => 4D:M 56:V
#define ISVOLUME	0x4D56

// surface index
#define SURFACE1	0x01
#define SURFACE2	0x02
#define SURFACE3	0x04
#define SURFACE4	0x08
#define SURFACE5	0x10
#define SURFACE6	0x20

// max number of recent files
#define MAX_RECENT_FILES	4

// voxel memory size
#define VOXEL_MEMORY_SIZE	3	// BYTE + WORD


/****************/
/* enumerations */
/****************/

// opengl view color type
enum
{ 
	WHITE_BLACK_COLOR = 1,
	BLACK_WHITE_COLOR,
	SINGLE_COLOR,
	INDEX_COLOR
};

// surface export file type
enum
{ 
	SRF_STL_ASCII=1,
	SRF_STL_BINARY,
	SRF_ACAD_SCRIPT,
	SRF_ACAD_DXF
};

// view type
enum
{
	NONE_VIEW = 0,
	PLAN_VIEW,
	OPENGL_VIEW,
	RENDER_VIEW,
	MCUBE_VIEW,
	MONTAGE_VIEW
};


/**************/
/* structures */
/**************/

// for 24bit bitmap ; don't use palette
typedef struct
{
	BYTE    rgbBlue;		// intensity of blue color
	BYTE    rgbGreen;		// intensity of green color
	BYTE    rgbRed;			// intensity of red color
} RGB;

// two-dimensional position coordinate
typedef struct
{
	WORD  x;
	WORD  y;
} POSITION_TWOD;

// three-dimensional position coordinate
typedef struct
{
	WORD  x;
	WORD  y;
	WORD  z;
} POSITION_THREED;

// rectangular region
typedef struct
{
	POSITION_TWOD  start_ele;

	WORD           width;
	WORD           height;

	WORD           depth_down;
	WORD           depth_up;
} REGION;

// keyword file structure
typedef struct
{
	QStringList  strList;
	QString      strImageFormat;

	REGION       regRegion;

	float        fVoxelWidth;
	float        fVoxelHeight;
	float        fVoxelDepth;

	bool         bWhiteIsBone;	// if true white voxel is bone, else black one is bone
} KEYWORD;

// rendering info for volpack
typedef struct
{
	double voxel_opacity[2];

	double mat_ambient[3];
	double mat_diffuse[3];
	double mat_specular[3];
	double mat_shinyness;

	double light_direction[3];
	double light_color[3];

	double depth_cueing[2];

	double rotate_angle[3];

	double view_port[6];
} RENDERINFO;

// 3d surface node coordinate
typedef struct
{
	float x;
	float y;
	float z;
} SRF_NODE;

// 3d finite element node
typedef struct
{
	DWORD id;

	float x;
	float y;
	float z;

	DWORD count;

	// Flag for node whether the node
	// is duplicated between
	// lower mesh's Upper Nodes
	// to Upper mesh's Lower Nodes or not
	bool bDupFlag;
} FE_NODE;

// finite element structure
typedef struct
{
	DWORD id;

	DWORD node[8];

	/*
	float fYoung;
	float fPassion;
	float fValue;
	*/

} FE_ELEMENT;

// volume structure
typedef struct{
	WORD   vfVersion;		// file version
	WORD   vfType;			// file type "VX"
	DWORD  vfSize;			// file size
} VOLUMEFILEHEADER;

typedef struct{
	WORD   viSize;			// structure size

	WORD   viWidth;			// volume width   : x-direction
	WORD   viHeight;		// volume height  : y-direction
	WORD   viDepth;			// volume depth   : z-direction

	float  fDx;				// voxel size in width-direction  (= element width)
	float  fDy;				// voxel size in height-direction (= element height)
	float  fDz;				// voxel size in depth-direction  (= element depth)

	float  fOriginDx;		// original voxel size in width-direction
	float  fOriginDy;		// original voxel size in height-direction
	float  fOriginDz;		// original voxel size in depth-direction

//	RGB    rgbThreshold;

//	DWORD  viTotalNumVoxel;	// total number of voxel in volume
} VOLUMEINFOHEADER;

typedef struct{
	WORD   wValue;
	BYTE   byteFlag;
} VOXEL;

typedef struct{
	VOLUMEFILEHEADER VolumeFileHeader;
	VOLUMEINFOHEADER VolumeInfoHeader;

	WORD  *wDensity;
	BYTE  *byteFlag;

	// VOXEL *pVoxel;
} VOLUMESTRUCT;


#endif // _DEFINE_H_
